#!/usr/bin/env python

"""
  generate f' and f'' near x-ray resonances for an atom, including
  fine-structure due to solid-state effects (ie, xafs/dafs).

 algorithm summary:
 1 the brennan-cowan implementation of the cromer-libermann (cl)
   calculation is used as a starting set of a causal f' and f''.
   modifications were made to the bc code, mostly to make it easier
   to use, more closely f77 conforming, and smaller.  this data is
   convolved with a lorenztian, typically with a width of a few ev.

 2 an externally supplied file containing the xafs mu(e) is used to
   improve f''.  special support for xmu.dat files from feff is
   provided, or the external file can contain *measured* mu(e) for
   the system of interest.  a simple matching procedure is done to
   make the supplied mu(e) match the f'' from cl.

 3 a differential kramers-kronig transform is used to convert the
   changes in f'' (ie f''_supplied - f''_cl) into the changes in  f'
   (ie f' - f'_cl).   the kk transform is done using a maclaurin
   series method, as suggested in the literature.

  the result is a causal pair of f' and f'' that reflect the
  presence of the atoms neighboring the central atom.

  --  Further notes on the algorithms used and instructions for  --
  --  program use are given in the program documentation.        --

  copyright 1997,...,2003  matt newville

  acknowledgements: julie cross, chuck bouldin, john rehr, and
            bruce ravel contributed to the design of this code.
            the best ideas were theirs.  all mistakes are mine.

  1.21 using ifeffit atomic data for core-width if ewidth=0 on input

"""

import Ifeffit
import dkinp, kkmclf, kkmclr

def messag(s):
    print s

def guess_iz(energy, expdat, npts, e0):
    # what is this?
    return 3

def main():
    versn = 1.3
    iff = Ifeffit.Ifeffit()
    # print version number
    messag(" -- diffkk version %s --" % versn)
    # read diffk.inp
    inp_info = dkinp.dkinp()
    # read mu(e) data, convert to f''(e) on an even energy grid
    messag(" Reading experimental data:  %s" % inp_info['xmufil'])
    iff.ifeffit("read_data(file='%s',group=dat,type=raw)" % inp_info['xmufil'])
    msg = iff.get_string('column_label')
    if "--undefined--" in msg:
        messag("  diffkk: fatal error.")
        return
    iff.ifeffit("set dat.energy = dat.%s" %  inp_info['iencol'])
    iff.ifeffit("set dat.expdat = dat.%s" % inp_info['imucol'])
    energy = iff.get_array('dat.energy')
    expdat = iff.get_array('dat.expdat')
    npts = len(expdat)
    e0 = float(inp_info['e0'])
    iatz = int(inp_info['iatz'])

    if iatz <= 2:
        iatz = guess_iz(energy, expdat, npts, e0)

    iff.put_scalar('e0',  e0)
    iff.put_scalar('egrid',  float(inp_info['egrid']))
    iff.put_scalar('elow',  float(inp_info['elow']))
    iff.put_scalar('ehigh',  float(inp_info['ehigh']))
    iff.ifeffit('set ework1 = floor(dat.energy)')
    iff.ifeffit('set ework2 = ceil(dat.energy)')
    iff.ifeffit('set eout1  = floor(dat.energy) - elow')
    iff.ifeffit('set eout2  = ceil(dat.energy) + ehigh')
    iff.ifeffit('set w.energy = range(ework1,ework2,egrid)')
    iff.ifeffit('set o.e1     = range(eout1,ework1-egrid,egrid)')
    iff.ifeffit('set o.e2     = range(ework2+egrid,eout2,egrid)')
    iff.ifeffit('set o.energy = range(eout1,eout2,egrid)')
    iff.ifeffit('set w.expdat = linterp(dat.energy,dat.expdat,w.energy)')

    energy = iff.get_array('w.energy')
    expdat = iff.get_array('w.expdat')
    npts = len(expdat)

    # generate initial tables of f' f'' on the same energy grid
    atz= iatz * 1.0
    iff.put_scalar('iz', atz)
    iff.ifeffit('f1f2(energy=o.e1, z=iz)')

    iff.ifeffit('set wid = core_width')
    if (inp_info['ewidth'] > 0):
        iff.put_scalar('wid', inp_info['ewidth'])

    iff.ifeffit('f1f2(energy=w.energy, z=iz,width=wid)')
    
    iff.ifeffit('set w.expscal =w.expdat')
    messag(" hello expdat is simple! ")
    
    expdat = iff.get_array('w.expscal')
    npts = len(expdat)

    iff.ifeffit('f1f2(energy=o.e1, z=iz,width=wid)')
    iff.ifeffit('set o.f1_low = o.f1')
    iff.ifeffit('set o.f2_low = o.f2')

    iff.ifeffit('f1f2(energy=o.e2, z=iz,width=wid)')
    iff.ifeffit('set o.f1_high = o.f1')
    iff.ifeffit('set o.f2_high = o.f2')

    #broaden f1cl and f2cl
    f1cl = iff.get_array('w.f1')
    f2cl = iff.get_array('w.f2')
    iff.ifeffit('set w.f1_cl = w.f1')
    iff.ifeffit('set w.f2_cl = w.f2')

    #align/shift/scale improved f''  to tabulated f''(df2 = expdat - f2cl)
    # call chrdmp(' Matching experimental data to Cromer-Libermann ')
    if inp_info['f2tof1']:
        msg = " f'' "
    else:
        msg = " f' "
    messag(" Matching experimental data to Cromer-Libermann %s" % msg)

    df2 = [0.0]*len(f2cl)
    for i in range(npts):
        df2[i] = -f2cl[i] + expdat[i]

   #do kk transform of delta f''  -> delta f'
    messag(' Doing difference Kramers-Kronig transform')
    if inp_info['f2tof1']:
        df1 = kkmclr.kkmclr(energy, df2)
    else:
        df1 = kkmclf.kkmclf(energy, df2)
       
    #add delta f' to initial f', delta f''  to initial f''

    o1 = [0]*npts
    o2 = [0]*npts
    for i in range(npts):
        o1[i]  = df1[i] + f1cl[i]
        o2[i]  = df2[i] + f2cl[i]
    iff.put_array('w.f1', o1)
    iff.put_array('w.f2', o2)

    #construct output on wider range
    iff.ifeffit('set o.f1_cl = join(join(o.f1_low, w.f1_cl),o.f1_high)')
    iff.ifeffit('set o.f1 = join(join(o.f1_low, w.f1),o.f1_high)')
    iff.ifeffit('set o.f2_cl = join(join(o.f2_low, w.f2_cl),o.f2_high)')
    iff.ifeffit('set o.f2 = join(join(o.f2_low, w.f2),o.f2_high)')


    #play with docs (put user titles at top,
    #keep as many old doc lines as fit)
    iff.put_string('doc', inp_info['title'])
    if inp_info['active']:
        messag(' ')
        messag('  Ready to write out data file:')
        outfil = input('** output file name ')
    else:
        outfil = inp_info['outfil']

    msg = "write_data(file= %s,o.energy,o.f1,o.f2,o.f1_cl,o.f2_cl,$doc)" % outfil 
    iff.ifeffit(msg)
    iff.ifeffit('save diffkk.sav')
    # messag(' writing summary to diffkk.log')
    #   call dklog
    messag(' -- diffkk done -- ')
       
if __name__ == "__main__":
    main()
